/************************************************************************

 Module:      JTVRemote.cpp

 Project:     JTV

 Description: Eline TVMaster remote control driver.
              

 Copyright (c) 2002 by Ivan Kolin  <ivkolin@mail.ru>

 All rights reserved.

 You may not use any piece of the following code for commercial purposes
 without written permission from the author.

 You may freely copy and distribute the source code as long as it contains
 this copyright notice and the disclaimer. You may freely modify it.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.    IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
#include <unknwn.h>
#include <stdio.h>

#include "jtvrem.h"
#include "jtvbt8x8.h"
#include "btregs.h"
#include "jtvremsk.h"

// This driver corresponds to the JTVRemote Specification 1.2.
// Look at JTVRemSpec1x.zip file for additional information.

// Key codes from: Slider, <gabiana@mail.ru>
//                                    

// Firm    : Eline (http://www.eline-net.com/elv-tvm.shtml)
// Kit Type: ?  
// Mark    : Eline Vision 
// Model   : ELV-TVM (Eline Vision TVMaster)
// Tuner   :
// Standart: 
// System  : 

// BTTool.exe display:
// GPIO_Dma_Ctl - 0000c004; (0000c007 when TV window open)
// GPIO_Out_En  - 00001800;
// GPIO_Data    - remarked column (for pressed - released
//                keys and opened/closed JTV-window).
// When open JTV-window, GPIO_Data values can change it's
// values in regular way that can be used while driver creation.
// Example: 00xxxxxx - 00xxxxxc, where "x" - random value.
// GPIO_Data    - 0096b700 (when JTV-window is open)
// GPIO_Data    - 0096ff00 (when JTV-window is closed)


KeyDef keys[]=
{
//                                           --- JTV opened ---    --- JTV closed ---
// 1-st row keys layout                      Pressed    Released   Pressed   Released
        { 0x00, "Key_Power", "Power", },   //0096b600 - 0096b700 / ??      - ??
// No key
        { 0x80, "Key_MouseMode", "", },  //0096be80 - 0096b700
        { 0x40, "Key_TVFullScreen", "", }, //0096b640 - 0096b700
// 2-nd row keys layout
        { 0x48, "Key_ChannelUp", "ChannelUp", },  //0096b648 - 0096b700 ?---?
        { 0xc0, "Key_1", "1", },         //0096b6c0 - 0096b700
        { 0x20, "Key_2", "2", },         //0096b620 - 0096b700
        { 0xa0, "Key_3", "3", },         //0096b6a0 - 0096b700
// ...
        { 0xc8, "Key_ChannelDown", "ChannelDown", },//0096b6c8 - 0096b700 ?---?
        { 0xe0, "Key_4", "4", },         //0096b6e0 - 0096b700
        { 0x10, "Key_5", "5", },         //0096b610 - 0096b700
        { 0x90, "Key_6", "6", },         //0096b690 - 0096b700

        { 0x28, "Key_VolumeUp", "VolumeUp", },  //0096b628 - 0096b700
        { 0xd0, "Key_7", "7", },        //0096b6d0 - 0096b700
        { 0x30, "Key_8", "8", },        //0096b630 - 0096b700
        { 0xb0, "Key_9", "9", },        //0096b6b0 - 0096b700

        { 0xe8, "Key_VolumeDown", "VolumeDown", }, //0096b6e8 - 0096b700
        { 0xf8, "Key_Cancel", "", },     //0096b6f8 - 0096b700
        { 0xf0, "Key_0", "0", },         //0096b6f0 - 0096b700
        { 0x08, "Key_Enter", "Enter", }, //0096b608 - 0096b700

        { 0x60, "Key_Recall", "Recall", },    //0096b660 - 0096b700
        { 0x70, "Key_AirCableTV", "", },//0096b670 - 0096b700
        { 0x88, "Key_Video", "", },     //0096b688 - 0096b700
        { 0x50, "Key_Inc", "", },       //0096b650 - 0096b700

//        { 0x48, "Key_Left", "Left", },  //0096b648 - 0096b700 ?---? *
//        { 0xc8, "Key_Empty", "", },     //0096b6c8 - 0096b700 ?---? *
        { 0xa8, "Key_Right", "Right", },//0096b6a8 - 0096b700
        { 0x68, "Key_Dec", "", },       //0096b668 - 0096b700

        { 0xb8, "Key_Window", "", },   //0096b6b8 - 0096b700
        { 0x98, "Key_Down", "", },     //0096b698 - 0096b700
        { 0x58, "Key_Esc", "", },      //0096b658 - 0096b700
// No key

        { 0x38, "Key_TabRight", "", }, //0096b638 - 00f4e700
        { 0x78, "Key_TabLeft", "", },  //0096b678 - 00f4e700
        { 0xd8, "Key_Mute", "", },     //0096b6d8 - 00f4e700
        { 0x18, "Key_AVSource", "", }, //0096b618 - 00f4e700
};
typedef UINT (CALLBACK* LPFNDLLFUNC1)(DWORD,UINT);


DWORD dwParam1;
UINT  uParam2, uReturnVal;

INT first_delay=1000;	// default first delay for a key
INT repeat_delay=100;	// default repeat delay for a key

void log(const char* str, ...)
{
  va_list arglist;

  FILE* l=fopen("JTVRemote.log","a+t");
  if (l==NULL) return;
  va_start( arglist, str );
  vfprintf(l, str, arglist );
  va_end( arglist );
  fclose(l);

// Look at section [JTVRemote] in "jtv.ini" file, look for strings
// "KeyFirstDelay=" and "KeyRepeatDelay=", example:
//      [JTVRemote]
//      ............
//      KeyFirstDelay=800
//      KeyRepeatDelay=60
// If strings exist, get corresponded values into "first_delay"
// and "repeat_delay" variables, if not - keep default values above.

  first_delay=GetPrivateProfileInt("JTVRemote","KeyFirstDelay",first_delay,".\\jtv.ini");
  repeat_delay=GetPrivateProfileInt("JTVRemote","KeyRepeatDelay",repeat_delay,".\\jtv.ini");

}

class JTVRemote : public CRemoteDriver
{

public:

        JTVRemote(KeyDef *k, int count) : CRemoteDriver(k, count)
		{
			log("> Initializing Eline TVMaster remote module.\n");
            log("> Copyright (c) 2002 by Ivan Kolin, <ivkolin@mail.ru>.\n");
			log("> KeyFirstDelay=%d, KeyRepeatDelay=%d\n",first_delay,repeat_delay);
		};


        void PollingThread()
        {
                IBT8x8Device *bt;
                DWORD last_code=0xffffffff;;
                DWORD last_tick=GetTickCount();
   
                if (SUCCEEDED(device->QueryInterface(IID_IBT8x8Device, (void **)&bt)))
                {
                    DWORD value;
                    INT key_pause=20; // new key initial delay
					DWORD press_flag=0;
					//DWORD test_code=0;

                    bt->ReadBT(BT848_GPIO_DMA_CTL, &value);
                    bt->WriteBT(BT848_GPIO_DMA_CTL, value | 0xC004);

                    bt->ReadBT(BT848_GPIO_OUT_EN, &value);
                    bt->WriteBT(BT848_GPIO_OUT_EN, (value | 0x00001800));

                    bt->ReadBT(BT848_GPIO_DATA, &value);
                    bt->WriteBT(BT848_GPIO_DATA, value | 0x96B700);
                   
				  	   
                    while (WaitForSingleObject(hBreakEvent, 0) != WAIT_OBJECT_0)
                    {
                         Sleep(key_pause);

                         DWORD gpio_data=0;
                         DWORD value=0;
						 
                         bt->ReadBT(BT848_GPIO_DATA, &gpio_data);
 						 
                         value = (gpio_data &0x000000ff);
					  	 press_flag = ((gpio_data & 0x00000f00)>>8);
						 
						 // Test block begin - comment all while build checked out driver
						 //if (test_code != value)
						 //{
							//log(">> ---- Test information ---- \n");
							//log(">>      Tick_Counter: %d\n",GetTickCount());
							//log(">>         GPIO_DATA: %x\n",gpio_data);
							//log(">>  Current_Key_Code: %x\n",value);
							//log(">> Previous_Key_Code: %x\n",last_code);

							//log(">>    Press_Key_Flag: %x\n",press_flag);
							//test_code = value;
						 //}
           				 // Test block end

                         if ((press_flag == 0x6)|(press_flag == 0xe)) // any key is pressed
                         {	 
                             if (last_code == value)
                             {     // auto repeat for any pressed key
								   //log(">>    Press_Key_Auto: %x\n",value); //for test
                            	   key_pause = repeat_delay;
                             } 
                             else
                             {	   // first run for any pressed key
								   //log(">>   Press_Key_First: %x\n",value); //for test
                                   last_code = value;
                                   key_pause = first_delay;
                             }
                             AddStroke(last_code);                             
                         }
                         else  // no one key is pressed
                         {	  
                             if ((value!=last_code) && (last_code!=0xffffffff))
                             {
                                last_code = value;
                             }
							   
                             // Example_Begin 
                             // Released key activation when JTV-window absent
                             // and any key has been pressed before
                             // else if (((gpio_data &0x0000000f)!=0xc) && (last_code!=0xffffffff))
                             // {
                             // 	 AddStroke(value); // Activation for defined RKey_XX
                             // }
                             // Example_End    
								   
                                key_pause = 20;
                         };
				
                    };

                log("Eline TVMaster: Break event received. Bye!\n");

                bt->Release();
                };
        };
};

extern "C"
{
        __declspec(dllexport) IJTVRemoteControl *__stdcall GetInterfacePtr()
        {
                return new JTVRemote(keys, sizeof(keys) / sizeof(keys[0]));
        };
};


